@@ -1,5 +1,6 @@
 as_function <- function(expr, envir = parent.frame(), enclos = baseenv(), ...) {
-  eval(substitute(function() x, list(x = expr)), envir = envir, enclos = enclos, ...)
+  fun_expr <- substitute(function() x, list(x = expr))
+  eval(fun_expr, envir = envir, enclos = enclos, ...)
 }
 
 # Although the set of "base" packages rarely changes, it has happened
@@ -47,6 +48,32 @@ is_internal <- function(x) {
   any(grepl(".Internal", body, fixed = TRUE))
 }
 
+# Example: base::.C_R_removeTaskCallback
+is_native_symbol_info <- function(x) {
+  if (!inherits(x, "NativeSymbolInfo")) return(FALSE)
+  if (typeof(x) != "list") return(FALSE)
+  address <- x$address
+  if (!inherits(address, "RegisteredNativeSymbol")) return(FALSE)
+  TRUE
+}
+
+isPackageNamespace <- function(env) {
+  if (!is.environment(env)) return(FALSE)
+  name <- environmentName(env)
+  if (name == "base") return(TRUE)
+  packageName <- env$.packageName
+  if (identical(name, packageName)) return(TRUE)
+  if (!grepl("^package:", name)) return(FALSE)
+  (name %in% search())
+}
+
+# From future 1.18.0
+asPkgEnvironment <- function(pkg) {
+  name <- sprintf("package:%s", pkg)
+  if (!name %in% search()) return(emptyenv())
+  as.environment(name)
+}
+
 ## From R.utils 2.0.2 (2015-05-23)
 hpaste <- function(..., sep="", collapse=", ", last_collapse=NULL,
                    max_head=if (missing(last_collapse)) 3 else Inf,
@@ -135,6 +162,7 @@ envname <- function(env) {
         capture.output(print(env))
       })	
     }
+    if (length(name) > 1L) name <- name[1]
     name <- gsub("(.*: |>)", "", name)
   } else {
     ## e.g. globals:::where("plan")
@@ -218,4 +246,52 @@ list_apply <- function(X, FUN, ...) {
   }
   res
 }
-	
\ No newline at end of file
+
+
+.trace <- new.env()
+.trace$indent <- 0L
+
+trace_indent <- function(x = "", indent = .trace$indent) {
+#  utils::str(list(indent = indent))
+#  indent <- max(0L, indent)
+  prefix <- paste(rep(" ", times = 3*indent), collapse = "")
+  paste(prefix, x, sep = "")
+}
+
+trace_printf <- function(..., indent = .trace$indent, collapse = "\n", appendLF = FALSE) {
+  msg <- sprintf(...)
+  out <- trace_indent(msg, indent = indent)
+  out <- paste(out, collapse = collapse)
+  message(out, appendLF = appendLF)
+  invisible(msg)
+}
+
+#' @importFrom utils capture.output
+trace_print <- function(..., envir = parent.frame(), indent = .trace$indent, collapse = "\n", appendLF = TRUE) {
+  bfr <- eval(capture.output(print(...)), envir = envir)
+  trace_printf(bfr, indent = indent, collapse = collapse, appendLF = appendLF)
+}
+
+#' @importFrom utils capture.output str
+trace_str <- function(..., envir = parent.frame(), indent = .trace$indent, collapse = "\n", appendLF = TRUE) {
+  bfr <- eval(capture.output(str(...)), envir = envir)
+  trace_printf(bfr, indent = indent, collapse = collapse, appendLF = appendLF)
+}
+
+trace_enter <- function(..., appendLF = TRUE) {
+  msg <- trace_printf(..., appendLF = FALSE)
+  message(" ...", appendLF = appendLF)
+  .trace$indent <- .trace$indent + 1L
+  attr(msg, "indent") <- .trace$indent
+  invisible(msg)
+}
+
+trace_exit <- function(fmtstr, ..., appendLF = TRUE) {
+  indent <- attr(fmtstr, "indent")
+  if (!is.null(indent)) .trace$indent <- indent
+  .trace$indent <- .trace$indent - 1L
+  msg <- trace_printf(fmtstr, ..., appendLF = FALSE)
+  message(" ... done", appendLF = appendLF)
+#  stop_if_not(.trace$indent >= 0L)
+  invisible(msg)
+}
